@@ -3,8 +3,8 @@
 #   Program:   Insight Segmentation & Registration Toolkit
 #   Module:    $RCSfile: Build.tcl,v $
 #   Language:  Tcl
-#   Date:      $Date: 2002/08/01 12:12:30 $
-#   Version:   $Revision: 1.50 $
+#   Date:      $Date: 2004/07/21 12:35:05 $
+#   Version:   $Revision: 1.78 $
 # 
 
 # Copyright (c) 2001 Insight Consortium
@@ -49,6 +49,7 @@
 
 proc Build { Model BuildStampDir } {
   global Dart
+
   set UtilityDir [file join $Dart(DartRoot) Source Client]
   source [file join $UtilityDir Utility.tcl]
 
@@ -163,7 +164,7 @@ proc Build { Model BuildStampDir } {
   foreach mkCommand $myMakeCommands {
       incr buildNumber
       puts "\n\tBuild command \#$buildNumber"
-      set processFile [open "| $mkCommand |& [list $Dart(TclshCommand)] [file join $Dart(DartRoot) Source Client Cat.tcl]"]
+      set processFile [open "| $mkCommand |& [list $Dart(TclshCommand)] \"[file join $Dart(DartRoot) Source Client Cat.tcl]\""]
       ReportProgress $processFile $outFile $Verbose
 
       puts $Out "<BuildCommand>[XMLSafeString $mkCommand]</BuildCommand>"
@@ -193,6 +194,14 @@ proc Build { Model BuildStampDir } {
   set Report(Error) 0
   set Report(Warning) 0
   set Report(FileMatched) 0
+  set RepeatCache("") 1
+  if { [info exists Dart(DetectRepeatedErrors)] && [string toupper $Dart(DetectRepeatedErrors)] == "ON" } \
+  {
+    set DetectRepeats 1
+  } else \
+  {
+    set DetectRepeats 0
+  }
 
   # Regular expressions used to determine if there is an error or warning
   # on a build log line
@@ -203,6 +212,8 @@ proc Build { Model BuildStampDir } {
 		    {([^ :]+):([0-9]+): ([^ \t])} \
 		    {([^:]+): error[ \t]*[0-9]+[ \t]*:} \
 		    {^Error ([0-9]+):} \
+		    {^Fatal } \
+		    {^Error: } \
 		    {^Error } \
                     {^\"[^\"]+\", line [0-9]+: [^Ww]} \
 		    {^cc[^C]*CC: ERROR File = ([^,]+), Line = ([0-9]+)} \
@@ -211,10 +222,20 @@ proc Build { Model BuildStampDir } {
 		    {([^ :]+) : (error|fatal error|catastrophic error)} \
 		    {([^:]+): (Error:|error|undefined reference|multiply defined)} \
 		    {([^:]+)\(([^\)]+)\) : (error|fatal error|catastrophic error)} \
+		    {^fatal error C[0-9]+:} \
+                    {: syntax error } \
+                    {^collect2: ld returned 1 exit status}\
 		    {Unsatisfied symbols:} \
+                    {Undefined symbols:} \
                     {^Undefined[ \t]+first referenced} \
 		    {^CMake Error:} \
-		    {:[ \t]cannot find}
+		    {:[ \t]cannot find} \
+		    {:[ \t]Can\'t find} \
+                    {: \*\*\* No rule to make target \`.*\'.  Stop} \
+                    {: Invalid loader fixup for symbol} \
+                    {: internal link edit command failed} \
+                    {^Fatal: Error detected} \
+                    {: Unrecognized option \`.*\'}
 		   ]
   set ErrorExceptions [list "instantiated from " "candidates are:" ": warning" "makefile:" "Makefile:" {:[ \t]+Where:}]
 
@@ -225,11 +246,13 @@ proc Build { Model BuildStampDir } {
 		      {([^:]+): warning ([0-9]+):} \
 		      {^\"[^\"]+\", line [0-9]+: [Ww]arning} \
   		      {([^:]+): warning[ \t]*[0-9]+[ \t]*:} \
+  		      {([^:]+): remark \#[0-9]+:} \
 		      {^Warning ([0-9]+):} \
 		      {^Warning } \
-                      {: syntax error } \
 		      {([^ :]+) : warning} \
-		      {([^:]+): warning}
+		      {([^:]+): warning} \
+		      {^This workspace file is corrupted} \
+                      {: Warnung:}
 		     ]
 
   # Need to ignore some makefile exceptions
@@ -246,9 +269,13 @@ proc Build { Model BuildStampDir } {
         {bind_at_load} \
         {XrmQGetResource} \
         {IceFlush} \
-        {warning LNK4089: all references to .GDI32.dll. discarded by .OPT:REF} \
-        {warning LNK4089: all references to .USER32.dll. discarded by .OPT:REF} \
-        {ld32: WARNING 85: definition of dataKey in}
+        {WARNING File = /usr/include/CC} \
+        {WARNING File = /usr/include/internal} \
+        {WARNING File = .*/vxl/vnl/algo/vnl_powell.cxx, Line = 140} \
+        {WARNING File = .*/vxl/vnl/algo/vnl_lbfgs.cxx, Line = 76} \
+        {warning LNK4089: all references to [^ \t]+ discarded by /OPT:REF} \
+        {ld32: WARNING 85: definition of dataKey in} \
+        {cc: warning 422: Unknown option \"\+b\.\" ignored} 
      ]
 
   puts "\tParsing Build Log"
@@ -262,112 +289,144 @@ proc Build { Model BuildStampDir } {
     set hasError [Match $ErrorMatches $LogLine $ErrorExceptions]
     set hasWarning [Match $WarningMatches $LogLine $WarningExceptions]
 
+    if { $hasWarning } \
+    {
+      set Type Warning
+      incr Report(Warning)
+    }
+
     if { $hasError } \
     {
       set Type Error
       incr Report(Error)
     }
     
-    if { $hasWarning } \
-    {
-      set Type Warning
-      incr Report(Warning)
-    }
     if { $hasError || $hasWarning } \
     {
-      set i $ErrorList(Counter)
-      incr ErrorList(Counter)
-      set ErrorList($i,Type) $Type
-      set ErrorList($i,Text) $LogLine
-      set ErrorList($i,Line) $Line
-      set ErrorList($i,PreContext) $ErrorList(Buffer)
-      set ErrorList($i,PostContext) ""
-      set ErrorList($i,ContextCounter) 0
-      set ErrorList($i,SourceFile) ""
-      set ErrorList($i,SourceLineNumber) ""
+      set isRepeat 0
+      if { $DetectRepeats } \
+      {
+	# puts "LOG LINE $logline"
 
-      # Take a stab at getting a file name out of the line
-      #
+	# Do this in pure Tcl
+        if { [info exists RepeatCache($LogLine)] } {
+          set j $RepeatCache($LogLine)
+	  set isRepeat 1
+	  incr ErrorList($j,RepeatCount)
+        }
+      }
 
-      # patch some paths to escape characters that are part of the regular
-      # expression syntax
-      regsub -all {\+} $Dart(SourceDirectory) {\+} sourceDirectory
-      regsub -all {\*} $sourceDirectory {\*} sourceDirectory
-      regsub -all {\^} $sourceDirectory {\^} sourceDirectory
-      regsub -all {\$} $sourceDirectory {\$} sourceDirectory
-      regsub -all {\[} $sourceDirectory {\[} sourceDirectory
-      regsub -all {\]} $sourceDirectory {\]} sourceDirectory
-      regsub -all {\(} $sourceDirectory {\)} sourceDirectory
-      regsub -all {\(} $sourceDirectory {\)} sourceDirectory
+      if { ! $isRepeat } \
+      {
+        set i $ErrorList(Counter)
+        incr ErrorList(Counter)
+        if { $DetectRepeats } {
+          set RepeatCache($LogLine) $i
+        }
+        set ErrorList($i,Type) $Type
+        set ErrorList($i,Text) $LogLine
+        set ErrorList($i,Line) $Line
+        set ErrorList($i,PreContext) $ErrorList(Buffer)
+        set ErrorList($i,PostContext) ""
+        set ErrorList($i,ContextCounter) 0
+        set ErrorList($i,SourceFile) ""
+        set ErrorList($i,SourceLineNumber) ""
+        set ErrorList($i,RepeatCount) 0
+      
+        # Take a stab at getting a file name out of the line
+        #
 
-      regsub -all {\+} $Dart(SourceDirectoryBase) {\+} sourceDirectoryBase
-      regsub -all {\*} $sourceDirectoryBase {\*} sourceDirectoryBase
-      regsub -all {\^} $sourceDirectoryBase {\^} sourceDirectoryBase
-      regsub -all {\$} $sourceDirectoryBase {\$} sourceDirectoryBase
-      regsub -all {\[} $sourceDirectoryBase {\[} sourceDirectoryBase
-      regsub -all {\]} $sourceDirectoryBase {\]} sourceDirectoryBase
-      regsub -all {\(} $sourceDirectoryBase {\)} sourceDirectoryBase
-      regsub -all {\(} $sourceDirectoryBase {\)} sourceDirectoryBase
+        # patch some paths to escape characters that are part of the regular
+        # expression syntax
+        regsub -all {\+} $Dart(SourceDirectory) {\+} sourceDirectory
+        regsub -all {\*} $sourceDirectory {\*} sourceDirectory
+        regsub -all {\^} $sourceDirectory {\^} sourceDirectory
+        regsub -all {\$} $sourceDirectory {\$} sourceDirectory
+        regsub -all {\[} $sourceDirectory {\[} sourceDirectory
+        regsub -all {\]} $sourceDirectory {\]} sourceDirectory
+        regsub -all {\(} $sourceDirectory {\)} sourceDirectory
+        regsub -all {\(} $sourceDirectory {\)} sourceDirectory
 
-      # For GCC, we get /path/to/source.cxx:##
-      # or ../../../Insight/Code/Common
-      set Found 0
-      if { !$Found && [regexp "$sourceDirectory/(\[^:\]*):(\[0-9\]+)" $LogLine dummy SourceFile LineNumber] } \
-      {
-	set ErrorList($i,SourceFile) $SourceFile
-	set ErrorList($i,SourceLineNumber) $LineNumber
-	set Found 1
-      }
-      if { !$Found && [regexp "$sourceDirectoryBase/(\[^:\]*):(\[0-9\]+)" $LogLine dummy SourceFile LineNumber] } \
-      {
-	set ErrorList($i,SourceFile) $SourceFile
-	set ErrorList($i,SourceLineNumber) $LineNumber
-	set Found 1
-      }
+        regsub -all {\+} $Dart(SourceDirectoryBase) {\+} sourceDirectoryBase
+        regsub -all {\*} $sourceDirectoryBase {\*} sourceDirectoryBase
+        regsub -all {\^} $sourceDirectoryBase {\^} sourceDirectoryBase
+        regsub -all {\$} $sourceDirectoryBase {\$} sourceDirectoryBase
+        regsub -all {\[} $sourceDirectoryBase {\[} sourceDirectoryBase
+        regsub -all {\]} $sourceDirectoryBase {\]} sourceDirectoryBase
+        regsub -all {\(} $sourceDirectoryBase {\)} sourceDirectoryBase
+        regsub -all {\(} $sourceDirectoryBase {\)} sourceDirectoryBase
+
+        # For GCC, we get /path/to/source.cxx:##
+        # or ../../../Insight/Code/Common
+        set Found 0
+        if { !$Found && [regexp "$sourceDirectory/(\[^:\]*):(\[0-9\]+)" $LogLine dummy SourceFile LineNumber] } \
+        {
+	  set ErrorList($i,SourceFile) $SourceFile
+	  set ErrorList($i,SourceLineNumber) $LineNumber
+	  set Found 1
+        }
+        if { !$Found && [regexp "$sourceDirectoryBase/(\[^:\]*):(\[0-9\]+)" $LogLine dummy SourceFile LineNumber] } \
+        {
+	  set ErrorList($i,SourceFile) $SourceFile
+	  set ErrorList($i,SourceLineNumber) $LineNumber
+	  set Found 1
+        }
       
-      # Sun CC
-      if { !$Found && [regexp "\"+$sourceDirectory/(\[^:\]*)\"+:(\[0-9\]+)" $LogLine dummy SourceFile LineNumber] } \
-      {
-	set ErrorList($i,SourceFile) $SourceFile
-	set ErrorList($i,SourceLineNumber) $LineNumber
-	set Found 1
-      }
+        # Sun CC
+        if { !$Found && [regexp "\"+$sourceDirectory/(\[^:\]*)\"+:(\[0-9\]+)" $LogLine dummy SourceFile LineNumber] } \
+        {
+	  set ErrorList($i,SourceFile) $SourceFile
+	  set ErrorList($i,SourceLineNumber) $LineNumber
+	  set Found 1
+        }
       
-      # MS VC++ 6.0
-      set match "${sourceDirectory}(\\\\|/)(\[^\(\]*)\\((\[0-9\]+)\\) : (error|fatal error|catastrophic error|warning)"
-      # Replace all the backslashes in the log line with forward slashes
-      regsub -all {\\} $LogLine {/} foo
-      if { !$Found && [regexp $match $foo dummy dummy2 SourceFile LineNumber] } \
-      {
-	# Replace all backslashes with forward slashes
-	regsub -all {\\} $SourceFile {/} dummy
-	set SourceFile $dummy
-	set ErrorList($i,SourceFile) $SourceFile
-	set ErrorList($i,SourceLineNumber) $LineNumber
-	set Found 1
-      }
+        # MS VC++ 6.0
+        set match "${sourceDirectory}(\\\\|/)(\[^\(\]*)\\((\[0-9\]+)\\) : (error|fatal error|catastrophic error|warning)"
+        # Replace all the backslashes in the log line with forward slashes
+        regsub -all {\\} $LogLine {/} foo
+        if { !$Found && [regexp $match $foo dummy dummy2 SourceFile LineNumber] } \
+        {
+	  # Replace all backslashes with forward slashes
+	  regsub -all {\\} $SourceFile {/} dummy
+	  set SourceFile $dummy
+	  set ErrorList($i,SourceFile) $SourceFile
+	  set ErrorList($i,SourceLineNumber) $LineNumber
+	  set Found 1
+        }
       
-      # Irix compiler
-      # cc-1552 CC: WARNING File = /home/blezek/src/Insight/Code/Common/itkAffineMutualInformationVW.txx, Line = 250
-      if { !$Found && [regexp "$sourceDirectory/(\[^,\]*), Line = (\[0-9\]+)" $LogLine dummy SourceFile LineNumber] } \
-      {
-	set ErrorList($i,SourceFile) $SourceFile
-	set ErrorList($i,SourceLineNumber) $LineNumber
-	set Found 1
-      }
+        # Irix compiler
+        # cc-1552 CC: WARNING File = /home/blezek/src/Insight/Code/Common/itkAffineMutualInformationVW.txx, Line = 250
+        if { !$Found && [regexp "$sourceDirectory/(\[^,\]*), Line = (\[0-9\]+)" $LogLine dummy SourceFile LineNumber] } \
+        {
+	  set ErrorList($i,SourceFile) $SourceFile
+	  set ErrorList($i,SourceLineNumber) $LineNumber
+	  set Found 1
+        }
+
+	# Try the BCC compiler
+	# Warning W8004 C:\itkQuality\Borland\Insight\Utilities\zlib\inftrees.c 137: 'p' is assigned a value that is never used in function huft_build
+	set match "${sourceDirectory}/(\[^ \]*) (\[0-9\]+):"
+        # Replace all the backslashes in the log line with forward slashes
+        regsub -all {\\} $LogLine {/} foo
+        if { !$Found && [regexp $match $foo dummy SourceFile LineNumber] } {
+	  # Replace all backslashes with forward slashes
+	  set ErrorList($i,SourceFile) $SourceFile
+	  set ErrorList($i,SourceLineNumber) $LineNumber
+	  set Found 1
+	}
+	
+        # CMake errors
+        # CMake Error: foo garf
+        if { !$Found && [regexp "^CMake Error:" $LogLine dummy] } \
+        {
+	  # No line numbers or source files are indicated by a CMake error
+	  set Found 1
+        }
       
-      # CMake errors
-      # CMake Error: foo garf
-      if { !$Found && [regexp "^CMake Error:" $LogLine dummy] } \
-      {
-	# No line numbers or source files are indicated by a CMake error
-	set Found 1
+        incr Report(FileMatched) $Found
+        lappend ErrorList(Append) $i
       }
-      
-      incr Report(FileMatched) $Found
-      lappend ErrorList(Append) $i
     }
-  
     # Add to the PreContextBuffer
     set l [llength $ErrorList(Buffer)]
     if { $l > $BufferLength } \
@@ -403,8 +462,14 @@ proc Build { Model BuildStampDir } {
   
   
   # Write all the errors out to the Out file
-  puts "\t$Report(Error) Compiler Errors"
-  puts "\t$Report(Warning) Compiler Warnings"
+  if { $Report(Error) > $Dart(BuildErrorReportLimit) && $Dart(BuildErrorReportLimit) >= 0} {
+    set Suppressed "Not reporting [expr $Report(Error) - $Dart(BuildErrorReportLimit)]"
+  } else { set Suppressed "" }
+  puts "\t$Report(Error) Compiler Errors $Suppressed"
+  if { $Report(Warning) > $Dart(BuildWarningReportLimit) && $Dart(BuildWarningReportLimit) >= 0 } {
+    set Suppressed "Not reporting [expr $Report(Warning) - $Dart(BuildWarningReportLimit)]"
+  } else { set Suppressed "" }
+  puts "\t$Report(Warning) Compiler Warnings $Suppressed"
   set total [expr $Report(Error) + $Report(Warning)]
   if { $total != 0 } \
   {
@@ -412,8 +477,24 @@ proc Build { Model BuildStampDir } {
     puts "\tParsed $Report(FileMatched) filenames from [expr $Report(Error) + $Report(Warning)] opportunities for $percent%"
   }
   puts "\tWriting XML"
+  set ReportedErrors 0
+  set ReportedWarnings 0
   for { set i 0 } { $i < $ErrorList(Counter) } { incr i } \
   {
+    switch $ErrorList($i,Type) {
+      Error {
+        if { $ReportedErrors > $Dart(BuildErrorReportLimit) && $Dart(BuildErrorReportLimit) >= 0 } {
+          continue
+        }
+        incr ReportedErrors
+      }
+      Warning {
+        if { $ReportedWarnings > $Dart(BuildWarningReportLimit) && $Dart(BuildErrorReportLimit) >= 0 } {
+          continue
+        }
+        incr ReportedWarnings
+      }
+    }
     puts $Out "\t<$ErrorList($i,Type)>"
     puts $Out "\t\t<BuildLogLine>[XMLSafeString $ErrorList($i,Line)]</BuildLogLine>"
     puts $Out "\t\t<Text>[XMLSafeString $ErrorList($i,Text)]"
@@ -436,6 +517,8 @@ proc Build { Model BuildStampDir } {
       puts $Out [XMLSafeString $l]
     }
     puts $Out "</PostContext>"
+    puts $Out "\t\t<RepeatCount>[XMLSafeString $ErrorList($i,RepeatCount)]</RepeatCount>"
+    
     puts $Out "</$ErrorList($i,Type)>"
     puts $Out "\n"
   }
