@@ -1,11 +1,11 @@
 # =========================================================================
-# 
+#
 #   Program:   Insight Segmentation & Registration Toolkit
 #   Module:    $RCSfile: Utility.tcl,v $
 #   Language:  Tcl
-#   Date:      $Date: 2002/06/07 10:47:24 $
-#   Version:   $Revision: 1.28 $
-# 
+#   Date:      $Date: 2005/10/03 19:58:16 $
+#   Version:   $Revision: 1.48 $
+#
 
 # Copyright (c) 2001 Insight Consortium
 # All rights reserved.
@@ -47,9 +47,10 @@ proc ManglePathName { path } {
   return $path
 }
 
+
 #
 # combines a command and a stdout/stderr redirect to a file in a safe manner
-#  
+#
 proc MakeCommand {cmd outDir } {
    set cmd2 "exec $cmd >& \"$outDir\""
    return $cmd2
@@ -70,8 +71,8 @@ proc LoadConfigurationFile { arrayname filename } {
     set line [string trim $line]
     # Ignore comments, empty lines, and mis-formed lines
     if { [string index $line 0] == "\#"
-	 || $line == ""
-	 || [string first ":" $line] == -1 } {
+     || $line == ""
+     || [string first ":" $line] == -1 } {
       continue
     }
     set index [string first ":" $line]
@@ -90,6 +91,19 @@ proc LoadConfigurationFile { arrayname filename } {
   }
 }
 
+proc SaxonOptions { args } \
+{
+  set targs {}
+  foreach a $args {
+      set ta [split $a =]  
+      if { [llength $ta] > 1 && [lindex $ta 1] != {} } {
+          lappend targs $a
+      }
+  }
+          
+  return $targs
+}
+
 proc MakeDateTimeStamp {} \
 {
   return [clock format [clock seconds] -format %Y%m%d-%H%M -gmt 1]
@@ -98,22 +112,43 @@ proc MakeDateTimeStamp {} \
 proc NeedToRemake { File } \
 {
   set XML [file join XML $File.xml]
-  if { ![file exists $File.html] } \
+  if { ![file exists $File.html] && ![file exists $File.htmlz]} \
   {
     # puts "Remake 1"
     return 1
   }
-  
-  if { [file exists $XML] } \
-  {
-    # puts "Remake: [expr [file mtime $File.html] <= [file mtime $XML]]"
-    return [expr [file mtime $File.html] <= [file mtime $XML]]
+
+  # If both uncompressed and compressed versions of the HTML file exist, use
+  # the date of the newest
+  set htmlFileMTime 0
+  set compressedHTMLFileMTime 0
+  set latestMTime 0
+
+  if { [file exists $File.html] } {
+     set htmlFileMTime [file mtime $File.html]
   }
 
+  if { [file exists $File.htmlz] } {
+     set compressedHTMLFileMTime [file mtime $File.htmlz]
+  }
+
+  if { [expr $htmlFileMTime > $compressedHTMLFileMTime] } {
+      set latestMTime $htmlFileMTime
+  } else {
+      set latestMTime $compressedHTMLFileMTime
+  }
+
+  # Is the XML file in XML directory?  i.e. something submitted by a client
+  if { [file exists $XML] } \
+  {
+    return [expr $latestMTime <= [file mtime $XML]]
+  }
+
+  # Is the file in the current directory? i.e. something produced on server
+  # while processing other XML files.
   if { [file exists $File.xml] } \
   {
-    # puts "Remake: [expr [file mtime $File.html] <= [file mtime $File.xml]]"
-    return [expr [file mtime $File.html] <= [file mtime $File.xml]]
+    return [expr $latestMTime <= [file mtime $File.xml]]
   }
   return 1
 }
@@ -131,7 +166,7 @@ proc IsBuildStampInDay { NightlyDateTimeStamp BuildStamp } \
   # NTS is in seconds, so we want the range to be from NTS to NTS + 24 hours
   set Start $NTS
   set End [expr $Start + 24 * 60 * 60]
-  
+
   if { [catch { set BSTS [ConvertStampToSeconds $BuildStamp] } r ] } \
   {
     puts $r
@@ -159,13 +194,13 @@ proc GetNightlySeconds {} \
     # on the date of the current client machine.  As such, this time
     # may be in the past or in the future.
     set nightlySeconds [clock scan $Dart(NightlyStartTime)]
-    
+
     # If nightlySeconds is in the past, this is the current open
     # dashboard, then return nightlySeconds.  If nightlySeconds is in
     # the future, this is the next dashboard to be opened, so subtract 24
     # hours to get the time of the current open dashboard
-    if { $nightlySeconds <= [clock seconds] } { 
-        return $nightlySeconds 
+    if { $nightlySeconds <= [clock seconds] } {
+        return $nightlySeconds
     } else {
         return [expr $nightlySeconds - 24 * 60 * 60]
     }
@@ -215,9 +250,12 @@ proc ConvertStampToSeconds { Stamp } \
   return $Seconds
 }
 
+# Test should be an optional function that takes the name of a directory
+# of file and returns false if it should be ignored.
+
 proc FileMap { Queue PatternList Map {Test {}}} \
 {
-  
+
   while { [llength $Queue] != 0 } \
   {
     set Filename [lindex $Queue 0]
@@ -227,7 +265,7 @@ proc FileMap { Queue PatternList Map {Test {}}} \
     {
       if { [file tail $Filename] == "CVS" } \
       {
-	continue
+        continue
       }
       if {$Test != {}} {
          if {[$Test $Filename]} {
@@ -238,7 +276,7 @@ proc FileMap { Queue PatternList Map {Test {}}} \
       }
       if { $Status } \
       {
-	puts "Caught $Result"
+        puts "Caught $Result"
       }
       continue;
     }
@@ -246,7 +284,9 @@ proc FileMap { Queue PatternList Map {Test {}}} \
     {
       if { [string match $Pattern [file tail $Filename]] } \
       {
-	catch { eval $Map $Filename }
+        if {$Test == {} || [$Test $Filename]} {
+          catch { eval $Map $Filename }
+        }
       }
     }
   }
@@ -254,13 +294,21 @@ proc FileMap { Queue PatternList Map {Test {}}} \
 
 proc GetYesterdayDateTimeStamp { NightlyDateTimeStamp } \
 {
-  set ts [ConvertStampToSeconds $NightlyDateTimeStamp]
+  if { $NightlyDateTimeStamp == "" } {
+    set ts [clock seconds]
+  } else {
+    set ts [ConvertStampToSeconds $NightlyDateTimeStamp]
+  }
   return [clock format [expr $ts - 24 * 60 * 60] -format %Y%m%d-%H%M -gmt 1]
 }
 
 proc GetTomorrowDateTimeStamp { NightlyDateTimeStamp } \
 {
-  set ts [ConvertStampToSeconds $NightlyDateTimeStamp]
+  if { $NightlyDateTimeStamp == "" } {
+    set ts [clock seconds]
+  } else {
+    set ts [ConvertStampToSeconds $NightlyDateTimeStamp]
+  }
   return [clock format [expr $ts + 24 * 60 * 60] -format %Y%m%d-%H%M -gmt 1]
 }
 
@@ -283,7 +331,7 @@ proc GetLastBuildDirectory { {Model Experimental} } \
   set BuildStampDirs ""
   set BuildStampDirs [glob -nocomplain -- [file join $BuildNameDir $GlobString]]
   return [lindex [lsort $BuildStampDirs] end]
-}  
+}
 
 proc GetLastDashboardDirectory { {Model Experimental} } \
 {
@@ -301,29 +349,63 @@ proc GetLastDashboardDirectory { {Model Experimental} } \
   set DashboardDirs ""
   set DashboardDirs [glob -nocomplain -- [file join Testing HTML TestingResults Dashboard $GlobString]]
   return [lindex [lsort $DashboardDirs] end]
-}  
+}
+
+
+proc EchoFileName { file } \
+{
+  puts $file
+}
+
+
+proc RemoveFile { file } \
+{
+  file delete -force -- $file
+}
+
+
+proc CompressOldFile { file } \
+{
+  global AgeMTime
+  global Dart
+
+  if { $Dart(CompressionType) == "gzip" } {
+      if { [expr [file mtime $file] < $AgeMTime] } {
+          puts "\tCompressing $file"
+          CompressFile $file ${file}z
+          # if the compression worked, then remove the original
+          set compressedSize 0
+          catch {set compressedSize [file size ${file}z]}
+          if { [expr $compressedSize > 0] } {
+              RemoveFile $file
+          }
+      } else {
+          # puts "$file compression skipped"
+      }
+  }
+}
 
 proc CompressFile { In Out } \
 {
   global Dart
-  
+
   if { $In == $Out } \
   {
     error "CompressFile, can not compress a file to itself $In"
   }
   switch -glob $Dart(CompressionCommand) \
   {
-    *gzip \
+    *gzip* \
     {
       set Dart(CompressionType) gzip
       return [exec $Dart(CompressionCommand) -c $In > $Out]
     }
-    *compress \
+    *compress* \
     {
       set Dart(CompressionType) compress
       return [exec $Dart(CompressionCommand) -c $In > $Out]
     }
-    *zip \
+    *zip* \
     {
       set Dart(CompressionType) zip
       return [exec $Dart(CompressionCommand) - $In > $Out]
@@ -338,14 +420,14 @@ proc CompressFile { In Out } \
 
 proc XMLConcat { FP filename } \
 {
-  catch {	
+  catch {
     set f [open $filename r]
       while { ![eof $f] } {
-	set line [gets $f]
-	if { ![string match {<?xml*} $line] } \
-	 {
+    set line [gets $f]
+    if { ![string match {<?xml*} $line] } \
+     {
          puts $FP $line
-	 }
+     }
       }
     # puts -nonewline $FP [read $f]
     close $f
@@ -362,9 +444,34 @@ proc XMLSafeString { str } \
   regsub -all ">" $str {\&gt;} str
   # regsub -all "'" $str {%apos;} str
 
-  # need to remove 
-  if { [info tclversion] > 8.0 } { regsub -all "\0" $str "" str }
+  return [PrintableString $str]
+  return $str
+}
 
+# Set inside PrintableString
+set XMLCharMap ""
+proc PrintableString { str } \
+{
+  global XMLCharMap
+  # need to remove nulls, and > 0x80
+  if { [info tclversion] > 8.0 } { regsub -all "\0" $str "^BadChar^" str }
+  if { [info tclversion] > 8.2 } {
+    # remove all problem characters
+    while { ![string is ascii -failindex idx $str] } {
+      # Remove the bad character
+      set str [string replace $str $idx $idx "^BadChar^"]
+    }
+    if { $XMLCharMap == "" } {
+      # Remove < 0x20, aside from \t, \n, \r
+      # > 20 hex are OK, == 32 decimal
+      for { set t 0 } { $t < 32 } { incr t } {
+        if { $t == 9 || $t == 10 || $t == 13 } { continue }
+        lappend XMLCharMap [binary format c $t]
+        lappend XMLCharMap "^BadChar^"
+      }
+    }
+    set str [string map $XMLCharMap $str]
+  }
   return $str
 }
 
@@ -392,10 +499,10 @@ proc GetDirectories { dir } \
 proc InitBase64 {} \
 {
   global Base64
-  
+
   if { [info exists Base64(0)] } { return }
   set String "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
-  
+
   set count [string length $String]
   for { set i 0 } { $i < $count } { incr i } \
   {
@@ -408,12 +515,12 @@ proc _toU8 { num } \
 {
   return [expr ( $num + 0x100 ) % 0x100]
 }
- 
+
 proc Base64Encode { String } \
 {
   global Base64
   InitBase64
-  
+
   set length [string length $String]
   set result ""
   set index 0
@@ -431,15 +538,15 @@ proc Base64Encode { String } \
     incr index
 
     # puts "$current0 $current1 $current2"
-        
-    append result $Base64([expr $current0 >> 2])   
+
+    append result $Base64([expr $current0 >> 2])
     append result $Base64([expr (($current0 & 0x03) << 4) + ($current1 >> 4 )])
     append result $Base64([expr (($current1 & 0x0f ) << 2) + ($current2 >> 6) ])
     append result $Base64([expr $current2 & 0x3f])
     incr length -3
   }
 
-  
+
   if { $length != 0 } \
   {
     # puts "length = $length"
@@ -454,7 +561,7 @@ proc Base64Encode { String } \
       incr index
       append result $Base64([expr (($current0 & 0x03) << 4) + ($current1 >> 4)])
       append result $Base64([expr ($current1 & 0x0f) << 2])
-      append result $Base64(Pad)      
+      append result $Base64(Pad)
     } \
     else \
     {
@@ -488,7 +595,7 @@ proc Base64EncodeFileToFile { In Out } \
   set result ""
   set index 0
   set count 0
-  
+
   while { ![eof $In] } \
   {
     append String [read $In 2049]
@@ -506,7 +613,7 @@ proc Base64EncodeFileToFile { In Out } \
       incr index
 
       set result ""
-      append result $Base64([expr $current0 >> 2])   
+      append result $Base64([expr $current0 >> 2])
       append result $Base64([expr (($current0 & 0x03) << 4) + ($current1 >> 4 )])
       append result $Base64([expr (($current1 & 0x0f ) << 2) + ($current2 >> 6) ])
       append result $Base64([expr $current2 & 0x3f])
@@ -515,11 +622,11 @@ proc Base64EncodeFileToFile { In Out } \
       puts -nonewline $Out $result
       if { $count >= 78 } \
       {
-	puts $Out ""
-	set count 0
+    puts $Out ""
+    set count 0
       }
     }
-  }    
+  }
 
   set result ""
   if { $length != 0 } \
@@ -563,7 +670,7 @@ proc lremove { l item } \
   return $ret
 }
 
-      
+
 proc Grep { matchlist file { Exceptions {} } } \
 {
   set result ""
@@ -574,19 +681,19 @@ proc Grep { matchlist file { Exceptions {} } } \
     {
       if { [regexp $match $line] } \
       {
-	set E 1
-	foreach Exception $Exceptions \
-	{
-	  if { [regexp $Exception $line] } \
-	  {
-	    set E 0
-	  }
-	}
-	if { $E } \
-	{
-	  lappend result $line
-	  break
-	}
+    set E 1
+    foreach Exception $Exceptions \
+    {
+      if { [regexp $Exception $line] } \
+      {
+        set E 0
+      }
+    }
+    if { $E } \
+    {
+      lappend result $line
+      break
+    }
       }
     }
   }
@@ -603,10 +710,10 @@ proc FindTests { Path {clear 1}} \
   global TestList
   if { ![info exists TestList] } { set TestList "" }
   if { $clear } { set TestList "" }
-  
+
   set testCommands [GetTestfileTests]
   if { $testCommands != ""} {
-    # insert $Path to the list for this test and change how tests are run to a 
+    # insert $Path to the list for this test and change how tests are run to a
     # (cwd $Path; exec; cwd $Back)
     foreach test $testCommands {
        set testCommand [list [lindex $test 0] $Path ]
@@ -642,7 +749,7 @@ proc GetCMakeVariable { Variable } \
   regsub -all {\)} $C1 ")\n " C
   set Contents $C
 
-  set variableValue ""  
+  set variableValue ""
   foreach Line [split $Contents "\n"] \
   {
     if { [string first $Variable $Line] != -1 } \
@@ -672,7 +779,7 @@ proc GetTestfileTests {} \
   regsub -all {\)} $C1 ")\n " C
   set Contents $C
 
-  set variableValue ""  
+  set variableValue ""
   foreach Line [split $Contents "\n"] \
   {
     if { [string first ADD_TEST $Line] != -1 } \
@@ -708,7 +815,7 @@ proc GetTestfileSubdirs {} \
   regsub -all {\)} $C1 ")\n " C
   set Contents $C
 
-  set variableValue ""  
+  set variableValue ""
   foreach Line [split $Contents "\n"] \
   {
     if { [string first SUBDIRS $Line] != -1 } \
@@ -721,7 +828,7 @@ proc GetTestfileSubdirs {} \
   }
   return $variableValue
 }
-  
+
 
 # If the specified Microsoft Workspace has an ALL_BUILD project, return all
 # projects it (ALL_BUILD) depends on.
@@ -731,13 +838,13 @@ proc FindAllBuildDSPDepends { Workspace } {
 
   if { [file exists $Workspace] } {
     set f [open $Workspace r]
-    
+
     while { ![eof $f] } {
       set pname ""
 
       set Line [gets $f]
 
-      # Extract the project name from 
+      # Extract the project name from
       # 'Project: "PROJECT_NAME"=PROJECT_NAME.dsp - Package Owner <4>'
       regexp "^Project: \"(\[^\"\]*)\"=" $Line dummy pname
 
@@ -798,7 +905,8 @@ proc GetAuthorsFromUpdateSummary { filename } {
     }
   }
   close $f
-  return [FormatEmailAddress [lunique $Authors]]
+  set Email [FormatEmailAddress [lunique $Authors]]
+  return $Email
 }
 
 
@@ -822,6 +930,47 @@ proc GetErrorsWarningsFromBuildSummary { filename } {
   return [list $Errors $Warnings]
 }
 
+proc GetStatusFromConfigureSummary { filename } {
+  set Status 0
+  set sexpression {<ConfigureStatus>([^<]*)</ConfigureStatus>}
+  if { ![file exists $filename] } { return $Status }
+  set f [open $filename r]
+  while { ![eof $f] } {
+    set line [gets $f]
+    if { [regexp $sexpression $line foo a] } {
+      set Status $a
+    }
+  }
+  close $f
+  return $Status
+}
+
+
+proc GetCountsFromTestSummary { filename } {
+  set Passed 0
+  set NotRun 0
+  set Failed 0
+  set pexpression {<PassedCount>([^<]*)</PassedCount>}
+  set fexpression {<FailedCount>([^<]*)</FailedCount>}
+  set nexpression {<NotRunCount>([^<]*)</NotRunCount>}
+  if { ![file exists $filename] } { return [list $NotRun $Failed $Passed] }
+  set f [open $filename r]
+  while { ![eof $f] } {
+    set line [gets $f]
+    if { [regexp $pexpression $line foo a] } {
+      set Passed $a
+    }
+    if { [regexp $nexpression $line foo a] } {
+      set NotRun $a
+    }
+    if { [regexp $fexpression $line foo a] } {
+      set Failed $a
+    }
+  }
+  close $f
+  return [list $NotRun $Failed $Passed]
+}
+
 
 proc FormatEmailAddress { Addresses } {
   global Dart Users
@@ -840,6 +989,20 @@ proc FormatEmailAddress { Addresses } {
       # Try to slap on the DefaultContinuousDomain variable
       if { [info exists Dart(DefaultContinuousDomain)] && $Dart(DefaultContinuousDomain) != "" } {
         lappend r "$Address@$Dart(DefaultContinuousDomain)"
+      } elseif { [info exists Dart(CVSIdentToEmail)] && $Dart(CVSIdentToEmail) != "" } {
+        set AuthorEmail ""
+        foreach Rule $Dart(CVSIdentToEmail) {
+          if { [regsub [lindex $Rule 0] $Address [lindex $Rule 1] AuthorEmail] } {
+            break
+          } else {
+            set AuthorEmail {}
+          }
+        }
+        if {$AuthorEmail == ""} {
+          puts "\t\tERROR: Can't match $Address in CVSIdentToEmail"
+        } elseif {$AuthorEmail != "DO_NOT_EMAIL"} {
+          set r [concat $r $AuthorEmail]
+        }
       }
     }
   }
@@ -852,7 +1015,7 @@ proc GetUsersList {} {
   # Read in the user list
   set CheckFiles [list [file join $Dart(SourceDirectory) Documentation UserList.txt] \
                   [file join $Dart(SourceDirectory) CVSROOT users]]
-  
+
   foreach File $CheckFiles {
     if {[file exists $File]} {
       set f [open $File]
@@ -862,8 +1025,12 @@ proc GetUsersList {} {
         switch [llength $l] \
         {
           3 {
+            # No clue what this Insight style is, so I will just ignore it (ANDY)
+            ## Insight style
+            #set Users([lindex $l 0]) [lindex $l 2]
+            #puts "Insight style: Users([lindex $l 0]) [lindex $l 2]"
             # Insight style
-            set Users([lindex $l 0]) [lindex $l 2]
+            set Users([lindex $l 0]) [lindex $l 1]
           }
           2 {
             # CVSROOT style
@@ -886,3 +1053,64 @@ proc DumpOutput { p out } {
     puts -nonewline $out [read $p 1]
   }
 }
+
+# This code handles fileevents and idle callbacks
+proc UtilityReadFile { p } {
+  global done Output
+  if { [eof $p] } {
+    set done "Completed"
+  } else {
+    if { [gets $p line] != -1 } {
+      puts $Output "$line\n"
+    }
+  }
+}
+
+proc UtilityIdleTimeout {} {
+  global done
+  set done "Timeout"
+}
+
+proc RunTimedCommand { command Timeout LogFile } {
+  global done
+  global Dart
+  global Output
+  global tcl_platform
+  # Setup a timer for the test.
+  set Output [open $LogFile w]
+
+  set p [open "| $command |&  [list $Dart(TclshCommand)] [file join $Dart(DartRoot) Source Client Cat.tcl]"]
+  # If the gets operation blocks, then the tcl event loop
+  # will never get a chance to see the timeout.
+  fconfigure $p -blocking 0
+
+  fileevent $p readable [list UtilityReadFile $p]
+  set Callback [after [expr int($Timeout * 1000)] UtilityIdleTimeout]
+  # Wait for a timeout, or the command to complete
+  vwait done
+  # Cancel the after callback, and the fileevent.
+  after cancel $Callback
+  fileevent $p readable ""
+
+  # Assume we passed, until proven otherwise.
+  set Status 0
+  # If we timed out, kill the process, if we can.
+  if { $done == "Timeout" } {
+    set Status 1
+    foreach PID [pid $p] {
+      if { $tcl_platform(platform) == "windows" } {
+        # this is a windows system, use the kill command distributed w/Dart
+        catch { exec [file join $Dart(DartRoot) Source Utilities win32 bin kill.exe] $PID }
+      } else {
+        # this is a unix system, use standard kill command
+        catch { exec kill $PID }
+      }
+    }
+  }
+
+  close $Output
+  # Configure the command pipeline to be blocking before closing,
+  # to ensure that close with a useful exit status.
+  fconfigure $p -blocking 1
+  return [close $p]
+}
